library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity uart_clock is
    Port ( clk50m : in std_logic;
           res : in std_logic;
           baudrate : in std_logic_vector(3 downto 0);
           baud_clk_o : out std_logic;
           baud_clk16_o : out std_logic);
end uart_clock;

architecture Behavioral of uart_clock is

  constant divider_value 	: integer := 27;

  signal base_baud_clk  	: std_logic;
  signal baud_clk_16    	: std_logic;  -- fr rx zum samplen (16fach)
  signal baud_clk       	: std_logic;  -- fr tx 
  signal baud_count_max 	: std_logic_vector ( 7 downto 0); -- max 2048 => ca 452 baud min.
  signal baud_count     	: std_logic_vector ( 7 downto 0); -- baudraten vorteiler (aus 1.851 MHz)
  signal baud_count_tx		: std_logic_vector ( 3 downto 0);

	signal baud_clk_16_z1		: std_logic;
	signal base_baud_clk_z1	: std_logic;
	
begin

  baud_clk_o   	<= baud_clk;
	baud_clk16_o 	<= baud_clk_16;

	-- "baudraten" basis generator (57600 * 16 * 2) ca. 1.851 MHz
	process (clk50m, res)
	  variable divider_counter : integer;
	begin
	  if res = '1' then
	    divider_counter := 0;
	    base_baud_clk <= '0';      

	  elsif clk50m'event and clk50m = '1' then
	    divider_counter := divider_counter + 1;
	    if divider_counter = divider_value then
	      base_baud_clk <= '1';      
	      divider_counter := 0;
	    else
	      base_baud_clk <= '0';      
	    end if;
	  end if;
	end process;

	-- baudraten generator RX (16fache Baudrate)
	process (clk50m, res)
	begin
	  if res = '1' then
	    baud_count  			<= (others => '0');
	    baud_clk_16 			<= '0';   
			base_baud_clk_z1 	<= '0';   

	  elsif clk50m'event and clk50m = '1' then

			base_baud_clk_z1 <= base_baud_clk;

	    if base_baud_clk_z1 = '0' and base_baud_clk = '1' then
		    baud_count <= baud_count + '1';
		    if baud_count = baud_count_max then
		      baud_count  <= (others => '0');      
		      baud_clk_16 <= '1';
		    else
		      baud_clk_16 <= '0';      
		    end if;
			end if;
	  end if;
	end process;

	-- baudraten generator TX
	process (clk50m, res)
	begin
	  if res = '1' then
	    baud_count_tx  	<= (others => '0');
	    baud_clk 				<= '0'; 
			baud_clk_16_z1  <= '0';

	  elsif clk50m'event and clk50m = '1' then
			baud_clk_16_z1  <= baud_clk_16; 
		  if baud_clk_16_z1 = '0' and baud_clk_16 = '1' then
		    baud_count_tx <= baud_count_tx + '1';
		    if baud_count_tx = X"0" then
		      baud_clk <= '1';
		    else
		      baud_clk <= '0';      
		    end if;
			end if;
	  end if;
	end process;


	-- baudraten umschaltung
	process (clk50m, res)
	begin  
	  if res = '1' then
	    baud_count_max <= X"03"; -- stdmssig mit 38400 baud starten

	  elsif clk50m'event and clk50m = '1' then
	    if baud_clk = '1' then
	      case baudrate is
	        when X"0" => baud_count_max <= X"60"; -- ca 1200 Baud
	        when X"1" => baud_count_max <= X"30"; -- ca 2400 Baud
	        when X"2" => baud_count_max <= X"18"; -- ca 4800 Baud
	        when X"3" => baud_count_max <= X"0c"; -- ca 9600 Baud
	        when X"4" => baud_count_max <= X"06"; -- ca 19200 Baud
	        when X"5" => baud_count_max <= X"03"; -- ca 38400 Baud
	        when X"6" => baud_count_max <= X"02"; -- ca 57600 Baud
	        when others => NULL;
	      end case;
	    end if;
	  end if;
	end process;


end Behavioral;
