library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity uart_tx is
    Port ( clk50m : in std_logic;
					 clk_baud : in std_logic;
           res : in std_logic;
           data_in : in std_logic_vector(7 downto 0);
           trg : in std_logic;
           rdy : out std_logic;
           tx_out : out std_logic);
end uart_tx;

architecture Behavioral of uart_tx is

	signal tx_buffer 		: std_logic_vector (7 downto 0);
	signal state 				: std_logic_vector (3 downto 0);
	signal clk_baud_z1	: std_logic;
	signal trg_z1     	: std_logic;

begin

	process (clk50m, res, data_in, trg)
	begin
		if res = '1' then

			state 			<= (others => '0');
			tx_buffer 	<= (others => '0');
			tx_out 			<= '1';
			clk_baud_z1 <= '0';
			trg_z1			<= '0';

		elsif clk50m'event and clk50m = '1' then

			clk_baud_z1 <= clk_baud;
			trg_z1			<= trg;

			if state = X"0" then
				if trg_z1 = '0' and trg = '1' then
					state 		<= X"1";
					tx_buffer <= data_in;
					rdy				<= '0';
				else
					rdy				<= '1';
				end if;
				tx_out 		<= '1';
			end if;

			if clk_baud_z1 = '0' and clk_baud = '1' then
				case state is
					when X"1" 	=> tx_out <= '0';  						state <= X"2";	rdy				<= '0';
					when X"2" 	=> tx_out <= tx_buffer (0);  	state <= X"3";	rdy				<= '0';
					when X"3" 	=> tx_out <= tx_buffer (1);  	state <= X"4";	rdy				<= '0';
					when X"4" 	=> tx_out <= tx_buffer (2);  	state <= X"5";	rdy				<= '0';
					when X"5" 	=> tx_out <= tx_buffer (3);  	state <= X"6";	rdy				<= '0';
					when X"6" 	=> tx_out <= tx_buffer (4);  	state <= X"7";	rdy				<= '0';
					when X"7" 	=> tx_out <= tx_buffer (5);  	state <= X"8";	rdy				<= '0';
					when X"8" 	=> tx_out <= tx_buffer (6);  	state <= X"9";	rdy				<= '0';
					when X"9" 	=> tx_out <= tx_buffer (7);  	state <= X"a";	rdy				<= '0';
					when X"a" 	=> tx_out <= '1';            	state <= X"0";	rdy				<= '0';
					when others => NULL;
				end case;	
			end if;

--			if state = X"c" then
--				if trg = '0' then
--					state <= X"0";
--				end if;
--			end if;


		end if;
	end process;

end Behavioral;
