library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity uart_tx_test1 is
    Port ( clk50m 	: in	std_logic;
           res 			: in 	std_logic;
           data_in 	: in 	std_logic_vector (7 downto 0);
           tx_out 	: out std_logic;
           swin 		: in 	std_logic_vector (7 downto 0);
           pbtn 		: in  std_logic_vector (1 downto 0);
					 ledout	 	: out std_logic_vector (2 downto 0));
end uart_tx_test1;

architecture Behavioral of uart_tx_test1 is

	COMPONENT uart_clock
	PORT(
		clk50m : IN std_logic;
		res : IN std_logic;
		baudrate : IN std_logic_vector(3 downto 0);          
		baud_clk_o : OUT std_logic;
		baud_clk16_o : OUT std_logic
		);
	END COMPONENT;

	COMPONENT uart_tx
	PORT(
		clk50m : IN std_logic;
		clk_baud : IN std_logic;
		res : IN std_logic;
		data_in : IN std_logic_vector(7 downto 0);
		trg : IN std_logic;          
		rdy : OUT std_logic;
		tx_out : OUT std_logic
		);
	END COMPONENT;

	signal baud_clk : std_logic;
	signal pb_count : std_logic_vector (31 downto 0);
	signal tx_trg   : std_logic;
	signal tx_out_o : std_logic;

begin

	Inst_uart_clock: uart_clock PORT MAP(
		clk50m 				=> clk50m,
		res 					=> res,
		baudrate 			=> X"4",
		baud_clk_o 		=> baud_clk,
		baud_clk16_o 	=> open
	);

	Inst_uart_tx: uart_tx PORT MAP(
		clk50m 		=> clk50m,
		clk_baud 	=> baud_clk,
		res 			=> res,
		data_in 	=> swin,
		trg 			=> tx_trg,
		rdy 			=> ledout (0),
		tx_out 		=> tx_out_o
	);

	ledout (1) <= baud_clk;
	ledout (2) <= tx_out_o;

	tx_out 		 <= tx_out_o;


	-- entprellung (einfach)

	process (clk50m,res,pb_count,pbtn)
	begin
		if res = '1' then
			pb_count <= (others => '0');
			tx_trg   <= '0';
		elsif	clk50m'event and clk50m = '1' then
			if pbtn (0) = '1' then
				pb_count <= pb_count + '1';
				if pb_count > X"004c4b40" then
					tx_trg <= '1';
				else	
					tx_trg <= '0';
				end if;
			else
			  if pbtn (1) = '1' then
				  tx_trg <= '1';
				else
				  pb_count <= (others => '0');
					tx_trg <= '0';
				end if;
			end if;



		end if;

	end process;


end Behavioral;
